{% extends "page-twocol.html" %}
{% load kegweblib %}
{% load crispy_forms_tags %}

{% block title %}Log In to {{kbsite.settings.title}}{% endblock %}
{% block pagetitle %}Log In{% endblock %}

{% block col-1 %}
<div class="well">
  <form action="" method="POST">
  {% csrf_token %}
  {{ form|crispy }}
  <button type="submit" class="btn btn-success small"
      name="tweet-form-submit">Log In</button>
<p>
  <span style="font-size:0.8em;">
    (did you <a href="{% url "password-reset" %}">forget your username or password?</a>)
  </span>
</p>

{% endblock col-1 %}